package com.novell.gso.jmstest.jms.config;

import javax.annotation.Resource;
import javax.jms.ConnectionFactory;
import javax.jms.Topic;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.jms.listener.AbstractJmsListeningContainer;
import org.springframework.jms.listener.DefaultMessageListenerContainer;

import com.novell.gso.jmstest.jms.SimpleMessageListener;

/**
 * XML no more!
 * 
 * This class is scanned by the Spring annotation processor and is used instead of all the
 * crazy bean setup in the applicationContext.xml (or other spring xml files) 
 *
 * For a good reference, see:
 * http://nurkiewicz.blogspot.com/2011/01/spring-framework-without-xml-at-all.html
 * 
 * IMPORTANT: This requires cglib.jar to be on the same path and in the same classloader hierarchy 
 * as the Spring libraries
 */
@Configuration
public class ContextConfiguration {
	

	/**
	 * Inject our ConnectionFactory and create a Bean for it
	 * 
	 * Note: using 'mappedName' with 
	 * @Resource forces Spring to use JNDI instead of the normal lookup with 'name' in the 
	 * application context.
	 */
	@Resource(mappedName = "jms/myCF")
	private ConnectionFactory myCF;
	
	@Bean
	public ConnectionFactory myCF() {
		return myCF;
	}

	
	/**
	 * Inject our Topic and create a Bean for it
	 */
	@Resource(mappedName = "jms/myTopic")
	public Topic myTopic;
	
	@Bean
	public Topic myTopic() {
		return myTopic;
	}
	
	
	/**
	 * Create an asynchronous listener (wrapped in a Spring container) for our topic
	 */
	@Bean
	public AbstractJmsListeningContainer myTopicContainer() {
	    final DefaultMessageListenerContainer container = new DefaultMessageListenerContainer();
	    container.setConnectionFactory(myCF);
	    container.setDestination(myTopic);
	    
	    // Publish/Subscribe (Topic), otherwise point-to-point (Queue)
	    container.setPubSubDomain(true);
	    
	    // Make this topic durable, or try anyways..
	    container.setSubscriptionDurable(true);
	    container.setDurableSubscriptionName("myTopic");
	    
	    // Set our Listener class 
	    container.setMessageListener(simpleMessageListener());
	    
	    return container;
	}
	
	private SimpleMessageListener simpleMessageListener() {
	    final SimpleMessageListener listener = new SimpleMessageListener();
	    return listener;
	}
}
